@@ -46,6 +46,7 @@
  *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  */
+
 package com.openexchange.jsieve;
 
 import java.io.BufferedOutputStream;
@@ -57,21 +58,18 @@ import java.net.InetSocketAddress;
 import java.net.Socket;
 import java.net.UnknownHostException;
 import java.util.ArrayList;
+import java.util.List;
 import java.util.StringTokenizer;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-
 import com.openexchange.jsieve.exceptions.OXSieveHandlerException;
 import com.openexchange.jsieve.exceptions.OXSieveHandlerInvalidCredentialsException;
 
 /**
- * This class is used to deal with the communication with sieve. For a
- * description of the communication system to sieve see {@see <a
- * href="http://www.ietf.org/internet-drafts/draft-martin-managesieve-07.txt"
- * >http://www.ietf.org/internet-drafts/draft-martin-managesieve-07.txt</a>}
+ * This class is used to deal with the communication with sieve. For a description of the communication system to sieve see
+ * {@see <a href="http://www.ietf.org/internet-drafts/draft-martin-managesieve-07.txt">http://www.ietf.org/internet-drafts/draft-martin-managesieve-07.txt</a>}
  * 
  * @author <a href="mailto:dennis.sieben@open-xchange.com">Dennis Sieben</a>
  */
@@ -105,6 +103,16 @@ public class SieveHandler {
 
     private final static String SIEVE_LOGOUT = "LOGOUT" + CRLF;
 
+    private static final int UNDEFINED = -1;
+
+    private static final int OK = 0;
+
+    private static final int NO = 1;
+
+    /*-
+     * Member section
+     */
+
     private boolean AUTH = false;
 
     private String sieve_user = null;
@@ -126,67 +134,10 @@ public class SieveHandler {
     private BufferedOutputStream bos_sieve = null;
 
     private static Log log = LogFactory.getLog(SieveHandler.class);
-    
-    public class Capabilities {
 
-        private Boolean starttls = Boolean.FALSE;
+    private long mStart;
 
-        private String implementation = null;
-
-        private ArrayList<String> sieve = null;
-
-        private ArrayList<String> sasl = null;
-
-        public ArrayList<String> getSasl() {
-            return sasl;
-        }
-
-        public void setSasl(final ArrayList<String> sasl) {
-            this.sasl = sasl;
-        }
-
-        public void addSasl(final String saslcapa) {
-            if (null != sasl) {
-                sasl.add(saslcapa);
-            } else {
-                sasl = new ArrayList<String>();
-                sasl.add(saslcapa);
-            }
-        }
-
-        public ArrayList<String> getSieve() {
-            return sieve;
-        }
-
-        public void setSieve(final ArrayList<String> sieve) {
-            this.sieve = sieve;
-        }
-
-        public void addSieve(final String sievecapa) {
-            if (null != sieve) {
-                sieve.add(sievecapa);
-            } else {
-                sieve = new ArrayList<String>();
-                sieve.add(sievecapa);
-            }
-        }
-
-        public String getImplementation() {
-            return implementation;
-        }
-
-        public void setImplementation(final String implementation) {
-            this.implementation = implementation;
-        }
-
-        public Boolean getStarttls() {
-            return starttls;
-        }
-
-        public void setStarttls(final Boolean starttls) {
-            this.starttls = starttls;
-        }
-    }
+    private long mEnd;
 
     /**
      * SieveHandler use socket-connection to manage sieve-scripts.<br>
@@ -206,7 +157,6 @@ public class SieveHandler {
         sieve_host_port = port;
     }
 
-
     public SieveHandler(final String userName, final String authUserName, final String authUserPasswd, final String host, final int port) {
         sieve_user = userName;
         sieve_auth = authUserName;
@@ -216,27 +166,25 @@ public class SieveHandler {
 
     }
 
-    private long mStart;
-    private long mEnd;
     private void measureStart() {
         this.mStart = System.currentTimeMillis();
     }
-    
+
     private void measureEnd(final String method) {
         this.mEnd = System.currentTimeMillis();
-        if( log.isDebugEnabled() ) {
-            log.debug("SieveHandler."+ method + "() took " + (this.mEnd-this.mStart) + "ms to perform");
+        if (log.isDebugEnabled()) {
+            log.debug("SieveHandler." + method + "() took " + (this.mEnd - this.mStart) + "ms to perform");
         }
     }
-    
+
     /**
-     * Use this function to initialize the connection. It will get the welcome messages from the
-     * server, parse the capabilities and login the user.
+     * Use this function to initialize the connection. It will get the welcome messages from the server, parse the capabilities and login
+     * the user.
      * 
      * @throws IOException
      * @throws UnsupportedEncodingException
      * @throws OXSieveHandlerException
-     * @throws OXSieveHandlerInvalidCredentialsException 
+     * @throws OXSieveHandlerInvalidCredentialsException
      */
     public void initializeConnection() throws IOException, OXSieveHandlerException, UnsupportedEncodingException, OXSieveHandlerInvalidCredentialsException {
         measureStart();
@@ -249,31 +197,101 @@ public class SieveHandler {
          * Set timeout to 30sec
          */
         s_sieve.setSoTimeout(30000);
-        bis_sieve = new BufferedReader(new InputStreamReader(s_sieve.getInputStream(),"UTF-8"));
+        bis_sieve = new BufferedReader(new InputStreamReader(s_sieve.getInputStream(), "UTF-8"));
         bos_sieve = new BufferedOutputStream(s_sieve.getOutputStream());
 
-        if (getServerWelcome()) {
-            log.debug("Got welcome from sieve");
-        } else {
+        if (!getServerWelcome()) {
             throw new OXSieveHandlerException("No welcome from server", sieve_host, sieve_host_port);
         }
+        log.debug("Got welcome from sieve");
         measureEnd("getServerWelcome");
-        
+        /*
+         * Capabilities read; further communication dependent on capabilities
+         */
         measureStart();
-        final ArrayList<String> temp = capa.getSasl();
+        List<String> sasl = capa.getSasl();
         measureEnd("capa.getSasl");
-        
-        if (null != temp && temp.contains("PLAIN")) {
+
+        final boolean issueTLS = capa.getStarttls().booleanValue();
+
+        final StringBuilder commandBuilder = new StringBuilder(64);
+
+        if (issueTLS) {
+            /*-
+             * Switch to TLS and re-fetch capabilities
+             *
+             *
+             * Send STARTTLS
+             * 
+             * C: STARTTLS
+             * S: OK
+             * <TLS negotiation, further commands are under TLS layer>
+             * S: "IMPLEMENTATION" "Example1 ManageSieved v001"
+             * S: "SASL" "PLAIN"
+             * S: "SIEVE" "fileinto vacation"
+             * S: OK
+             */
             measureStart();
-            if (selectAuth("PLAIN")) {
-                log.debug("Authentication to sieve successful");
-            } else {
-                throw new OXSieveHandlerInvalidCredentialsException("Authentication failed");
+            bos_sieve.write(commandBuilder.append("STARTTLS").append(CRLF).toString().getBytes("UTF-8"));
+            bos_sieve.flush();
+            measureEnd("startTLS");
+            commandBuilder.setLength(0);
+            /*
+             * Expect OK
+             */
+            while (true) {
+                final String temp = bis_sieve.readLine();
+                if (null == temp) {
+                    throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+                } else if (temp.startsWith(SIEVE_OK)) {
+                    break;
+                } else if (temp.startsWith(SIEVE_AUTH_FAILD)) {
+                    throw new OXSieveHandlerException("can't auth to SIEVE ", sieve_host, sieve_host_port);
+                }
             }
-            measureEnd("selectAuth");
-        } else {
-            throw new OXSieveHandlerException("The server doesn't suppport PLAIN authentication", sieve_host, sieve_host_port);
+            /*
+             * Switch to TLS
+             */
+            s_sieve = SocketFetcher.startTLS(s_sieve, sieve_host);
+            bis_sieve = new BufferedReader(new InputStreamReader(s_sieve.getInputStream(), "UTF-8"));
+            bos_sieve = new BufferedOutputStream(s_sieve.getOutputStream());
+            /*
+             * Fire CAPABILITY command but only for cyrus that is not sieve draft conform to sent CAPABILITY response again directly as
+             * response for the STARTTLS command.
+             */
+            if (capa.getImplementation().startsWith("Cyrus") || capa.getImplementation().startsWith("NEMESIS")) {
+	            measureStart();
+	            bos_sieve.write(commandBuilder.append("CAPABILITY").append(CRLF).toString().getBytes("UTF-8"));
+	            bos_sieve.flush();
+	            measureEnd("capability");
+	            commandBuilder.setLength(0);
+            }
+            /*
+             * Read capabilities
+             */
+            measureStart();
+            if (!getServerWelcome()) {
+                throw new OXSieveHandlerException("No TLS negotiation from server", sieve_host, sieve_host_port);
+            }
+            measureEnd("tlsNegotiation");
+            sasl = capa.getSasl();
         }
+        /*
+         * Check for PLAIN authentication support
+         */
+        if (null == sasl || !sasl.contains("PLAIN")) {
+            throw new OXSieveHandlerException(
+                new StringBuilder(64).append("The server doesn't suppport PLAIN authentication over a ").append(
+                    issueTLS ? "TLS" : "plain-text").append(" connection.").toString(),
+                sieve_host,
+                sieve_host_port);
+        }
+        measureStart();
+        if (!selectAuth("PLAIN", commandBuilder)) {
+            throw new OXSieveHandlerInvalidCredentialsException("Authentication failed");
+        }
+        log.debug("Authentication to sieve successful");
+        measureEnd("selectAuth");
     }
 
     /**
@@ -281,11 +299,12 @@ public class SieveHandler {
      * 
      * @param script_name
      * @param script
+     * @param commandBuilder
      * @throws OXSieveHandlerException
      * @throws UnsupportedEncodingException
      * @throws IOException
      */
-    public void setScript(final String script_name, final byte[] script) throws OXSieveHandlerException, IOException, UnsupportedEncodingException {
+    public void setScript(final String script_name, final byte[] script, final StringBuilder commandBuilder) throws OXSieveHandlerException, IOException, UnsupportedEncodingException {
         if (AUTH == false) {
             throw new OXSieveHandlerException("Script upload not possible. Auth first.", sieve_host, sieve_host_port);
         }
@@ -294,7 +313,10 @@ public class SieveHandler {
             throw new OXSieveHandlerException("Script upload not possible. No Script", sieve_host, sieve_host_port);
         }
 
-        final String put = SIEVE_PUT + '\"' + script_name + "\" {" + script.length + "+}" + CRLF;
+        final String put = commandBuilder.append(SIEVE_PUT).append('\"').append(script_name).append("\" {").append(script.length).append(
+            "+}").append(CRLF).toString();
+        commandBuilder.setLength(0);
+
         bos_sieve.write(put.getBytes("UTF-8"));
         bos_sieve.write(script);
 
@@ -331,18 +353,18 @@ public class SieveHandler {
     }
 
     /**
-     * Activate/Deactivate sieve script. Is status is true, activate this
-     * script.
+     * Activate/Deactivate sieve script. Is status is true, activate this script.
      * 
      * @param script_name
      * @param status
+     * @param commandBuilder
      * @throws IOException
      * @throws UnsupportedEncodingException
      * @throws OXSieveHandlerException
      */
-    public void setScriptStatus(final String script_name, final boolean status) throws OXSieveHandlerException, UnsupportedEncodingException, IOException {
+    public void setScriptStatus(final String script_name, final boolean status, final StringBuilder commandBuilder) throws OXSieveHandlerException, UnsupportedEncodingException, IOException {
         if (status) {
-            activate(script_name);
+            activate(script_name, commandBuilder);
         } else {
             deactivate(script_name);
         }
@@ -361,13 +383,64 @@ public class SieveHandler {
         if (!AUTH) {
             throw new OXSieveHandlerException("Get script not possible. Auth first.", sieve_host, sieve_host_port);
         }
-        final String get = SIEVE_GET_SCRIPT + "\"" + script_name + "\"" + CRLF;
+        final StringBuilder sb = new StringBuilder(32);
+        final String get = sb.append(SIEVE_GET_SCRIPT).append('"').append(script_name).append('"').append(CRLF).toString();
         bos_sieve.write(get.getBytes("UTF-8"));
         bos_sieve.flush();
-        final StringBuilder sb = new StringBuilder();
+        sb.setLength(0);
+        /*-
+         * If the script does not exist the server MUST reply with a NO response. Upon success a string with the contents of the script is
+         * returned followed by a OK response.
+         * 
+         * Example:
+         * 
+         * C: GETSCRIPT "myscript"
+         * S: {54+}
+         * S: #this is my wonderful script
+         * S: reject "I reject all";
+         * S:
+         * S: OK
+         */
+        {
+            final String firstLine = bis_sieve.readLine();
+            if (null == firstLine) {
+                // End of the stream reached
+                throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+            }
+            final int[] parsed = parseFirstLine(firstLine);
+            final int respCode = parsed[0];
+            if (NO == respCode || OK == respCode) {
+                /*
+                 * Either received a NO or an OK which indicates end of response. In both cases no script is availabe.
+                 */
+                return "";
+            }
+            sb.ensureCapacity(parsed[1]);
+        }
+        while (true) {
+            final String temp = bis_sieve.readLine();
+            if (null == temp) {
+                throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+            }
+            if (temp.startsWith(SIEVE_OK)) {
+                if (sb.length() >= 2) {
+                    // We have to strip off the last trailing CRLF...
+                    return sb.substring(0, sb.length() - 2);
+                }
+                return sb.toString();
+            }
+            sb.append(temp);
+            sb.append(CRLF);
+        }
+        /*-
+         * 
+         * 
         boolean firstread = true;
         while (true) {
             final String temp = bis_sieve.readLine();
+            if (null == temp) {
+                throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+            }
             if (temp.startsWith(SIEVE_OK)) {
                 // We have to strip off the last trailing CRLF...
                 return sb.substring(0, sb.length() - 2);
@@ -383,6 +456,7 @@ public class SieveHandler {
                 sb.append(CRLF);
             }
         }
+         */
     }
 
     /**
@@ -405,6 +479,9 @@ public class SieveHandler {
         final ArrayList<String> list = new ArrayList<String>();
         while (true) {
             final String temp = bis_sieve.readLine();
+            if (null == temp) {
+                throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+            }
             if (temp.startsWith(SIEVE_OK)) {
                 return list;
             }
@@ -439,13 +516,16 @@ public class SieveHandler {
         String scriptname = null;
         while (true) {
             final String temp = bis_sieve.readLine();
+            if (null == temp) {
+                throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+            }
             if (temp.startsWith(SIEVE_OK)) {
                 return scriptname;
             }
             if (temp.startsWith(SIEVE_NO)) {
                 throw new OXSieveHandlerException("Sieve has no script list", sieve_host, sieve_host_port);
             }
-            
+
             if (temp.matches(".*ACTIVE")) {
                 scriptname = temp.substring(temp.indexOf('\"') + 1, temp.lastIndexOf('\"'));
             }
@@ -453,7 +533,6 @@ public class SieveHandler {
 
     }
 
-    
     /**
      * Remove the sieve script. If the script is active it is deactivated before removing
      * 
@@ -469,15 +548,22 @@ public class SieveHandler {
         if (null == script_name) {
             throw new OXSieveHandlerException("Script can't be removed", sieve_host, sieve_host_port);
         }
-        
-        setScriptStatus(script_name, false);
-        
-        final String delete = SIEVE_DELETE + "\"" + script_name + "\"" + CRLF;
+
+        final StringBuilder commandBuilder = new StringBuilder(64);
+
+        setScriptStatus(script_name, false, commandBuilder);
+
+        final String delete = commandBuilder.append(SIEVE_DELETE).append("\"").append(script_name).append("\"").append(CRLF).toString();
+        commandBuilder.setLength(0);
+
         bos_sieve.write(delete.getBytes("UTF-8"));
         bos_sieve.flush();
 
         while (true) {
             final String temp = bis_sieve.readLine();
+            if (null == temp) {
+                throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+            }
             if (temp.startsWith(SIEVE_OK)) {
                 return;
             } else if (temp.startsWith(SIEVE_NO)) {
@@ -502,11 +588,14 @@ public class SieveHandler {
         }
     }
 
-    private boolean getServerWelcome() throws UnknownHostException, IOException {
+    private boolean getServerWelcome() throws UnknownHostException, IOException, OXSieveHandlerException {
         capa = new Capabilities();
 
         while (true) {
             final String test = bis_sieve.readLine();
+            if (null == test) {
+                throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+            }
             if (test.startsWith(SIEVE_OK)) {
                 return true;
             } else if (test.startsWith(SIEVE_NO)) {
@@ -518,11 +607,18 @@ public class SieveHandler {
         }
     }
 
-    private boolean authPLAIN() throws IOException, UnsupportedEncodingException {
-        final String to64 = sieve_user + '\0' + sieve_auth + '\0' + sieve_auth_passwd;
-        final String user_auth_pass_64 = convertStringToBase64(to64) + CRLF;
-        final String auth_mech_string = SIEVE_AUTH + "\"PLAIN\" ";
-        final String user_size = "{" + (user_auth_pass_64.length() - 2) + "+}" + CRLF;
+    private boolean authPLAIN(final StringBuilder commandBuilder) throws IOException, UnsupportedEncodingException {
+        final String to64 = commandBuilder.append(sieve_user).append('\0').append(sieve_auth).append('\0').append(sieve_auth_passwd).toString();
+        commandBuilder.setLength(0);
+
+        final String user_auth_pass_64 = commandBuilder.append(convertStringToBase64(to64)).append(CRLF).toString();
+        commandBuilder.setLength(0);
+
+        final String auth_mech_string = commandBuilder.append(SIEVE_AUTH).append("\"PLAIN\" ").toString();
+        commandBuilder.setLength(0);
+
+        final String user_size = commandBuilder.append("{").append((user_auth_pass_64.length() - 2)).append("+}").append(CRLF).toString();
+        commandBuilder.setLength(0);
 
         // We don't need to specify an encoding here because all strings contain only ASCII Text
         bos_sieve.write(auth_mech_string.getBytes());
@@ -548,14 +644,19 @@ public class SieveHandler {
     }
 
     // FIXME: Not tested yet
-    private boolean authLOGIN() throws IOException, OXSieveHandlerException, UnsupportedEncodingException {
+    private boolean authLOGIN(final StringBuilder commandBuilder) throws IOException, OXSieveHandlerException, UnsupportedEncodingException {
+
+        final String auth_mech_string = commandBuilder.append(SIEVE_AUTH).append("\"LOGIN\"").append(CRLF).toString();
+        commandBuilder.setLength(0);
 
-        final String auth_mech_string = SIEVE_AUTH + "\"LOGIN\"" + CRLF;
         bos_sieve.write(auth_mech_string.getBytes("UTF-8"));
         bos_sieve.flush();
 
         while (true) {
             final String temp = bis_sieve.readLine();
+            if (null == temp) {
+                throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+            }
             if (temp.endsWith(SIEVE_AUTH_LOGIN_USERNAME)) {
                 break;
             } else if (temp.endsWith(SIEVE_AUTH_FAILD)) {
@@ -563,14 +664,21 @@ public class SieveHandler {
             }
         }
 
-        final String user64 = convertStringToBase64(sieve_auth) + CRLF;
-        final String user_size = '{' + (user64.length() - 2) + "+}" + CRLF;
+        final String user64 = commandBuilder.append(convertStringToBase64(sieve_auth)).append(CRLF).toString();
+        commandBuilder.setLength(0);
+
+        final String user_size = commandBuilder.append('{').append((user64.length() - 2)).append("+}").append(CRLF).toString();
+        commandBuilder.setLength(0);
+
         bos_sieve.write(user_size.getBytes("UTF-8"));
         bos_sieve.write(user64.getBytes("UTF-8"));
         bos_sieve.flush();
 
         while (true) {
             final String temp = bis_sieve.readLine();
+            if (null == temp) {
+                throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+            }
             if (temp.endsWith(SIEVE_AUTH_LOGIN_PASSWORD)) {
                 break;
             } else if (temp.endsWith(SIEVE_AUTH_FAILD)) {
@@ -578,14 +686,21 @@ public class SieveHandler {
             }
         }
 
-        final String pass64 = convertStringToBase64(sieve_auth_passwd) + CRLF;
-        final String pass_size = '{' + (pass64.length() - 2) + "+}" + CRLF;
+        final String pass64 = commandBuilder.append(convertStringToBase64(sieve_auth_passwd)).append(CRLF).toString();
+        commandBuilder.setLength(0);
+
+        final String pass_size = commandBuilder.append('{').append((pass64.length() - 2)).append("+}").append(CRLF).toString();
+        commandBuilder.setLength(0);
+
         bos_sieve.write(pass_size.getBytes("UTF-8"));
         bos_sieve.write(pass64.getBytes("UTF-8"));
         bos_sieve.flush();
 
         while (true) {
             final String temp = bis_sieve.readLine();
+            if (null == temp) {
+                throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+            }
             if (temp.startsWith(SIEVE_OK)) {
                 AUTH = true;
                 return true;
@@ -595,17 +710,22 @@ public class SieveHandler {
         }
     }
 
-    private void activate(final String sieve_script_name) throws OXSieveHandlerException, UnsupportedEncodingException, IOException {
+    private void activate(final String sieve_script_name, final StringBuilder commandBuilder) throws OXSieveHandlerException, UnsupportedEncodingException, IOException {
         if (AUTH == false) {
             throw new OXSieveHandlerException("Activate a script not possible. Auth first.", sieve_host, sieve_host_port);
         }
 
-        final String active = SIEVE_ACTIVE + '\"' + sieve_script_name + '\"' + CRLF;
+        final String active = commandBuilder.append(SIEVE_ACTIVE).append('\"').append(sieve_script_name).append('\"').append(CRLF).toString();
+        commandBuilder.setLength(0);
+
         bos_sieve.write(active.getBytes("UTF-8"));
         bos_sieve.flush();
 
         while (true) {
             final String temp = bis_sieve.readLine();
+            if (null == temp) {
+                throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+            }
             if (temp.startsWith(SIEVE_OK)) {
                 return;
             } else if (temp.startsWith(SIEVE_NO)) {
@@ -618,7 +738,7 @@ public class SieveHandler {
         if (AUTH == false) {
             throw new OXSieveHandlerException("Deactivate a script not possible. Auth first.", sieve_host, sieve_host_port);
         }
-        
+
         boolean scriptactive = false;
         if (sieve_script_name.equals(getActiveScript())) {
             scriptactive = true;
@@ -627,9 +747,12 @@ public class SieveHandler {
         if (scriptactive) {
             bos_sieve.write(SIEVE_DEACTIVE.getBytes("UTF-8"));
             bos_sieve.flush();
-            
+
             while (true) {
                 final String temp = bis_sieve.readLine();
+                if (null == temp) {
+                    throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port);
+                }
                 if (temp.startsWith(SIEVE_OK)) {
                     return;
                 } else if (temp.startsWith(SIEVE_NO)) {
@@ -646,11 +769,11 @@ public class SieveHandler {
      * @throws UnsupportedEncodingException
      * @throws OXSieveHandlerException
      */
-    private boolean selectAuth(final String auth_mech) throws IOException, UnsupportedEncodingException, OXSieveHandlerException {
+    private boolean selectAuth(final String auth_mech, final StringBuilder commandBuilder) throws IOException, UnsupportedEncodingException, OXSieveHandlerException {
         if (auth_mech.equals("PLAIN")) {
-            return authPLAIN();
+            return authPLAIN(commandBuilder);
         } else if (auth_mech.equals("LOGIN")) {
-            return authLOGIN();
+            return authLOGIN(commandBuilder);
         }
         return false;
     }
@@ -661,7 +784,6 @@ public class SieveHandler {
         final String sieve = "\"SIEVE\"";
         final String sasl = "\"SASL\"";
 
-        
         String temp = line;
 
         if (temp.startsWith(starttls)) {
@@ -695,17 +817,110 @@ public class SieveHandler {
     }
 
     /**
-     * 
      * @param toConvert
      * @return Base64String
      * @throws UnsupportedEncodingException
      */
     private String convertStringToBase64(final String toConvert) throws UnsupportedEncodingException {
-        final String converted = com.openexchange.tools.encoding.Base64.encode(toConvert.getBytes("UTF-8")); 
+        final String converted = com.openexchange.tools.encoding.Base64.encode(toConvert.getBytes("UTF-8"));
         return converted.replaceAll("(\\r)?\\n", "");
     }
 
+    /**
+     * Parses the first line of a SIEVE response.
+     * <p>
+     * Examples:<br>
+     * &nbsp;<code>{54+}</code><br>
+     * &nbsp;<code>No {31+}</code><br>
+     * 
+     * @param firstLine The first line
+     * @return An array of <code>int</code> with length 2. The first position holds the response code if any available ({@link #NO} or
+     *         {@link #OK}), otherwise {@link #UNDEFINED}. The second position holds the number of octets of a following literal or
+     *         {@link #UNDEFINED} if no literal is present.
+     */
+    private static int[] parseFirstLine(final String firstLine) {
+        if (null == firstLine) {
+            return null;
+        }
+        final int[] retval = new int[2];
+        retval[0] = UNDEFINED;
+        retval[1] = UNDEFINED;
+        // Check for starting "NO" or "OK"
+        final char[] chars = firstLine.toCharArray();
+        int index = 0;
+        if ('N' == chars[index] && 'O' == chars[index + 1]) {
+            retval[0] = NO;
+            index += 2;
+        } else if ('O' == chars[index] && 'K' == chars[index + 1]) {
+            retval[0] = OK;
+            index += 2;
+        }
+        // Check for a literal
+        if (index < chars.length) {
+            char c;
+            while ((index < chars.length) && (((c = chars[index]) == ' ') || (c == '\t'))) {
+                index++;
+            }
+            if (index < chars.length && '{' == chars[index]) {
+                // A literal
+                retval[1] = parseLiteralLength(readString(index, chars));
+            }
+        }
+
+        return retval;
+    }
+
+    private static final Pattern PAT_LIT_LEN = Pattern.compile("\\{([0-9]+)(\\+?)\\}");
+
+    private static int parseLiteralLength(final String respLen) {
+        final Matcher matcher = PAT_LIT_LEN.matcher(respLen);
+        if (matcher.matches()) {
+            try {
+                return Integer.parseInt(matcher.group(1));
+            } catch (final NumberFormatException e) {
+                log.error(e.getMessage(), e);
+                return -1;
+            }
+        }
+        return -1;
+    }
+
+    private static String readString(final int index, final char[] chars) {
+        final int size = chars.length;
+        if (index >= size) {
+            // already at end of response
+            return null;
+        }
+        // Read until delimiter reached
+        final int start = index;
+        int i = index;
+        char c;
+        while ((i < size) && ((c = chars[i]) != ' ') && (c != '\r') && (c != '\n') && (c != '\t')) {
+            i++;
+        }
+        return toString(chars, start, i);
+    }
+
+    /**
+     * Convert the chars within the specified range of the given byte array into a {@link String}. The range extends from <code>start</code>
+     * till, but not including <code>end</code>.
+     */
+    private static String toString(final char[] chars, final int start, final int end) {
+        final int size = end - start;
+        final char[] theChars = new char[size];
+        for (int i = 0, j = start; i < size;) {
+            theChars[i++] = (chars[j++]);
+        }
+        return new String(theChars);
+    }
+
+    /**
+     * Gets the capabilities.
+     * 
+     * @return The capabilities
+     */
     public Capabilities getCapabilities() {
         return this.capa;
     }
+
 }
