@@ -68,9 +68,9 @@ import com.openexchange.config.ConfigurationService;
 import com.openexchange.groupware.AbstractOXException;
 import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.ldap.UserStorage;
+import com.openexchange.jsieve.Capabilities;
 import com.openexchange.jsieve.SieveHandler;
 import com.openexchange.jsieve.SieveTextFilter;
-import com.openexchange.jsieve.SieveHandler.Capabilities;
 import com.openexchange.jsieve.SieveTextFilter.ClientRulesAndRequire;
 import com.openexchange.jsieve.SieveTextFilter.RuleListAndNextUid;
 import com.openexchange.jsieve.commands.ActionCommand;
@@ -452,8 +452,8 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
         final SieveHandler sieveHandler = connectRight(credentials);
         try {
             sieveHandler.initializeConnection();
-            final String activeScript = sieveHandler.getActiveScript();
-            final String script = sieveHandler.getScript(activeScript);
+            final String activeScript = fixParsingError(sieveHandler.getActiveScript());
+            final String script = fixParsingError(sieveHandler.getScript(activeScript));
             final RuleListAndNextUid rules = sieveTextFilter.readScriptFromString(script);
 
             final ClientRulesAndRequire clientrulesandrequire = sieveTextFilter.splitClientRulesAndRequire(rules
@@ -484,8 +484,7 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
         } catch (final IOException e) {
             throw new OXMailfilterException(Code.IO_CONNECTION_ERROR, e, EMPTY_ARGS);
         } catch (final OXSieveHandlerException e) {
-            throw new OXMailfilterException(Code.SIEVE_COMMUNICATION_ERROR, e, e.getSieveHost(), Integer.valueOf(e
-                    .getSieveHostPort()), credentials.getRightUsername(), credentials.getContextString());
+            handleParsingException(e, credentials);
         } catch (final OXSieveHandlerInvalidCredentialsException e) {
             throw new OXMailfilterException(Code.INVALID_CREDENTIALS, e, EMPTY_ARGS);
         } catch (final ParseException e) {
@@ -513,6 +512,11 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
         }
     }
 
+    private String fixParsingError(String script) {
+        String pattern = ":addresses\\s+:";
+        return script.replaceAll(pattern, ":addresses \"\" :");
+    }
+
     @Override
     protected void actionDeleteScript(final MailfilterRequest request) throws AbstractOXException {
         final Credentials credentials = request.getCredentials();
@@ -791,12 +795,14 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
      */
     private void writeScript(final SieveHandler sieveHandler, final String activeScript, final String writeback)
             throws OXSieveHandlerException, IOException, UnsupportedEncodingException {
+        final StringBuilder commandBuilder = new StringBuilder(64);
+        
         if (null != activeScript && activeScript.equals(this.scriptname)) {
-            sieveHandler.setScript(activeScript, writeback.getBytes("UTF-8"));
-            sieveHandler.setScriptStatus(activeScript, true);
+            sieveHandler.setScript(activeScript, writeback.getBytes("UTF-8"), commandBuilder);
+            sieveHandler.setScriptStatus(activeScript, true, commandBuilder);
         } else {
-            sieveHandler.setScript(this.scriptname, writeback.getBytes("UTF-8"));
-            sieveHandler.setScriptStatus(this.scriptname, true);
+            sieveHandler.setScript(this.scriptname, writeback.getBytes("UTF-8"), commandBuilder);
+            sieveHandler.setScriptStatus(this.scriptname, true, commandBuilder);
         }
     }
 
@@ -807,5 +813,21 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
         }
         return msg;
     }
-
+    
+    /**
+     * The SIEVE parser is not very expressive when it comes to exceptions.
+     * This method analyses an exception message and throws a more detailed
+     * one if possible.
+     */
+    private void handleParsingException(OXSieveHandlerException e, Credentials credentials) throws OXMailfilterException{
+        String message = e.toString();
+        
+        if(message.contains("unexpected SUBJECT"))
+            throw new OXMailfilterException(Code.EMPTY_MANDATORY_FIELD, e, "ADDRESS (probably)");
+        if(message.contains("address ''"))
+            throw new OXMailfilterException(Code.EMPTY_MANDATORY_FIELD, e, "ADDRESS");
+        
+        throw new OXMailfilterException(Code.SIEVE_COMMUNICATION_ERROR, e, e.getSieveHost(), Integer.valueOf(e
+            .getSieveHostPort()), credentials.getRightUsername(), credentials.getContextString());
+    }
 }
